#include "obexserversessionfiletransfer.h"
#include <QTimer>
#include <qdbusconnection.h>
#include <qdbusinterface.h>
#include <KDebug>
#include "obexserversession.h"
#include <solid/control/bluetoothmanager.h>
#include "obexserver.h"

ObexServerSessionFileTransfer::ObexServerSessionFileTransfer(ObexServerSession* serverSession, QDBusInterface* session, const QString& filename, const QString& path, qulonglong size) {
	m_serverSession = serverSession;
	m_dbusSession = session;
	m_fileName = filename;
	if(path.length() != 0)
		m_localPath = path;
	else
		m_localPath = m_serverSession->server()->path() + "/" + filename;
	m_totalFileSize = size;
	setCapabilities(Killable);
}


ObexServerSessionFileTransfer::~ObexServerSessionFileTransfer() {
}

void ObexServerSessionFileTransfer::start() {
	QTimer::singleShot(0, this, SLOT(receiveFiles()));
}

void ObexServerSessionFileTransfer::reject() {
	m_dbusSession->call("Reject");
	doKill();
}

void ObexServerSessionFileTransfer::receiveFiles() {
	kDebug() << m_serverSession->path();
	QMap<QString, QString> sessionInfo = m_serverSession->server()->getServerSessionInfo(QDBusObjectPath(m_serverSession->path()));
	kDebug() << sessionInfo["BluetoothAddress"];

	Solid::Control::BluetoothManager &bluetoothManager = Solid::Control::BluetoothManager::self();
	Solid::Control::BluetoothInterface* bluetoothInterface = new Solid::Control::BluetoothInterface(bluetoothManager.defaultInterface());
	Solid::Control::BluetoothRemoteDevice bluetoothDevice = bluetoothInterface->findBluetoothRemoteDeviceAddr(sessionInfo["BluetoothAddress"]);

	emit description(this, "Receiving file over bluetooth", QPair<QString, QString>("From", bluetoothDevice.name()), QPair<QString, QString>("To", m_localPath));

	QDBusConnection dbusConnection = m_dbusSession->connection();
	dbusConnection.connect("", "", m_dbusSession->interface(), "TransferProgress", this, SLOT(slotTransferProgress(qulonglong)));
	dbusConnection.connect("", "", m_dbusSession->interface(), "TransferCompleted", this, SLOT(slotTransferCompleted()));
	dbusConnection.connect("", "", m_dbusSession->interface(), "ErrorOccurred", this, SLOT(slotErrorOccured(QString,QString)));
	kDebug() << "Transfer started ...";
	setTotalAmount(Bytes, m_totalFileSize);
	setProcessedAmount(Bytes, 0);
	m_dbusSession->call("Accept");
}

void ObexServerSessionFileTransfer::slotTransferProgress(qulonglong transferred) {
	kDebug() << "Transfer progress ...";
	ulong currentPercent = percent();
	setProcessedAmount(Bytes, transferred);
	ulong newPercent = percent();
	qulonglong unit = m_totalFileSize / 100;
	emitSpeed(newPercent*unit - currentPercent*unit);
}

void ObexServerSessionFileTransfer::slotTransferCompleted() {
	kDebug() << "Transfer completed";
	setProcessedAmount(Bytes, m_totalFileSize);
	emitResult();
}

void ObexServerSessionFileTransfer::slotErrorOccured(const QString& , const QString& ) {
	kDebug() << "Transfer disconnected";
	emitResult();
}

bool ObexServerSessionFileTransfer::doKill() {
	m_dbusSession->call("Cancel");
	m_dbusSession->call("Disconnect");
	return true;
}

QString ObexServerSessionFileTransfer::fileName() {
	return m_fileName;
}

QString ObexServerSessionFileTransfer::localPath() {
	return m_localPath;
}

qulonglong ObexServerSessionFileTransfer::fileSize() {
	return m_totalFileSize;
}

