#include "obexsessionfiletransfer.h"
#include <QTimer>
#include <KJobUiDelegate>
#include <KIO/JobUiDelegate>

#include "obexsession.h"
#include <KDebug>
#include <solid/control/bluetoothinterface.h>
#include <solid/control/bluetoothmanager.h>

ObexSessionFileTransfer::ObexSessionFileTransfer(ObexSession* osession, QDBusInterface* dsession, const QString& path) {
	m_obexSession = osession;
	m_dbusSession = dsession;
	m_localPath = path;
	setCapabilities(Killable);
}

ObexSessionFileTransfer::~ObexSessionFileTransfer() {

}

bool ObexSessionFileTransfer::doKill() {
	kDebug() << "Transfer canceled ...";
	m_dbusSession->call("Cancel");
	return true;
}


void ObexSessionFileTransfer::start() {
	QTimer::singleShot(0, this, SLOT(sendFile()));
}

void ObexSessionFileTransfer::sendFile() {
	Solid::Control::BluetoothManager &bluetoothManager = Solid::Control::BluetoothManager::self();
	Solid::Control::BluetoothInterface* bluetoothInterface = new Solid::Control::BluetoothInterface(bluetoothManager.defaultInterface());
	Solid::Control::BluetoothRemoteDevice bluetoothDevice = bluetoothInterface->findBluetoothRemoteDeviceAddr(m_obexSession->targetAddress());

	emit description(this, "Sending file over bluetooth", QPair<QString, QString>("From", m_localPath), QPair<QString, QString>("To", bluetoothDevice.name()));

	QDBusConnection dbusConnection = m_dbusSession->connection();
	dbusConnection.connect(m_dbusSession->service(), m_dbusSession->path(), m_dbusSession->interface(), "TransferStarted", this, SLOT( slotTransferStarted(const QString&, const QString&, qulonglong) ));
	dbusConnection.connect(m_dbusSession->service(), m_dbusSession->path(), m_dbusSession->interface(), "TransferProgress", this, SLOT( slotTransferProgress(qulonglong) ));
	dbusConnection.connect(m_dbusSession->service(), m_dbusSession->path(), m_dbusSession->interface(), "TransferCompleted", this, SLOT( slotTransferCompleted() ));
	
	m_dbusSession->call("SendFile", m_localPath);
}

void ObexSessionFileTransfer::slotTransferStarted(const QString& filename, const QString& path, qulonglong totalSize) {
	kDebug() << "Transfer started ...";
	setTotalAmount(Bytes, totalSize);
	m_totalFileSize = totalSize;
	setProcessedAmount(Bytes, 0);
}

void ObexSessionFileTransfer::slotTransferProgress(qulonglong transferred) {
	kDebug() << "Transfer progress ...";
	ulong currentPercent = percent();
	setProcessedAmount(Bytes, transferred);
	ulong newPercent = percent();
	qulonglong unit = m_totalFileSize / 100;
	emitSpeed(newPercent*unit - currentPercent*unit);
}

void ObexSessionFileTransfer::slotTransferCompleted() {
	kDebug() << "Transfer completed";
	setProcessedAmount(Bytes, m_totalFileSize);
	emitResult();
}
